
import json
import boto3, botocore

def init_bucket(location, profile=None):
    location = location.replace('s3://', '')
    if profile is None:
        s3_resource = boto3.resource('s3')
        s3_client = boto3.client('s3')
    else:
        session = boto3.Session(profile_name=profile)
        s3_resource = session.resource('s3')
        s3_client = session.client('s3')
    # check if the bucket exists
    try:
        s3_resource.meta.client.head_bucket(Bucket=location)
        print(f'Bucket exists at {location}')
    except botocore.exceptions.ClientError as e:
        if '403' in str(e):
            print('That bucket name already exists in AWS, try a different and more unique name')
            exit()
        try:
            s3_resource.create_bucket(Bucket=location) 
            print(f'Created bucket "{location}"')
        except botocore.exceptions.ClientError as e:
            if 'ExpiredToken' in str(e):
                print('Your AWS token has expired, please request a new token')
                exit()
            else:
                raise e
        except botocore.errorfactory.BucketAlreadyExists:
            print('That bucket name already exists in AWS, try a different and more unique name')
            exit()
    authorized = False
    try:
        policy = json.loads(s3_client.get_bucket_policy(Bucket=location)['Policy'])
    except botocore.exceptions.ClientError:
        policy = {
            "Version": "2012-10-17",
            "Statement": []
        }
    for statement in policy['Statement']:
        if statement['Sid'] == 'MaxarArdS3Access':
            authorized = True
            break
    if not authorized:
        statement =  {
            "Sid":"MaxarARDS3Access",
            "Effect":"Allow",
            "Principal":{
                "AWS":"arn:aws:iam::334489843805:root"
            },
            "Action":"s3:PutObject",
            "Resource":[
                f"arn:aws:s3:::{location}/*",f"arn:aws:s3:::{location}"
                ]
            }
        policy['Statement'].append(statement)
        s3_client.put_bucket_policy(Bucket=location, Policy=json.dumps(policy))
        print('Added ARD writer policy to bucket')
    else:
        print('Bucket already authorized')
    response = s3_client.put_bucket_ownership_controls(
        Bucket=location,
        OwnershipControls={
            'Rules': [
                {
                    'ObjectOwnership': 'BucketOwnerPreferred'
                },
            ]
        }
    )
    print('Object Ownership set to Bucket Owner Preferred')


def revoke_bucket(location, profile=None):
    ''' Removes the bucket policy statement from LOCATION, preventing the ARD writer from writing to the bucket '''

    location = location.replace('s3://', '')
    if profile is None:
        s3_resource = boto3.resource('s3')
        s3_client = boto3.client('s3')
    else:
        session = boto3.Session(profile_name=profile)
        s3_resource = session.resource('s3')
        s3_client = session.client('s3')

    # check if the bucket exists
    try:
        s3_resource.meta.client.head_bucket(Bucket=location)
    except botocore.exceptions.ClientError:
        print(f"Can't find bucket {location}")
        exit()
    try:
        policy = json.loads(s3_client.get_bucket_policy(Bucket=location)['Policy'])
    except botocore.exceptions.ClientError:
        print('Bucket has no policy statements - nothing to revoke')
        exit()
    statements = []
    for statement in policy['Statement']:
        if statement['Sid'] != 'MaxarARDS3Access':
            statements.append(statement)
    if len(statements) < len(policy['Statement']):
        if len(statements) == 0:
            s3_client.delete_bucket_policy(Bucket=location)
        else:
            policy['Statement'] = statements
            s3_client.put_bucket_policy(Bucket=location, Policy=json.dumps(policy))
        print('Removed ARD writer policy from bucket')
    else:
        print('Could not find ARD writer policy in bucket')